import urllib.parse
import urllib.request
from typing import Any

from ..config import Config
from .base import Handler


class HEHandler(Handler):
    def __init__(self, zone_name: str, config: Config, token: str) -> None:
        super().__init__(zone_name, config, token)
        self.nameservers = ['ns1.he.net', 'ns2.he.net', 'ns3.he.net', 'ns4.he.net', 'ns5.he.net']
        self.password = config.get_single_secret(zone_name)

    def set_record(self, record_name: str, value: str) -> Any:
        full_record_name = record_name + '.' + self.zone
        req = urllib.request.Request('https://dyn.dns.he.net/nic/update')
        data = {
            'hostname': full_record_name,
            'password': self.password,
            'txt': value,
        }
        req.data = urllib.parse.urlencode(data).encode('utf-8')
        req.add_header('Content-Type', 'application/x-www-form-urlencoded')
        req.method = 'POST'
        try:
            with urllib.request.urlopen(req) as f:
                return f.read()
        except urllib.error.HTTPError as ex:
            self.log.error('cannot set the record %s: %s', record_name, ex)
            res = ex.fp.read().decode('utf-8')
            raise Exception(res) from ex

    def create(self, record_name: str, record_value: str) -> None:
        self.log.info('creating %s with value %s', record_name, record_value)
        self.set_record(record_name, record_value)

    def remove(self, record_name: str) -> None:
        self.log.info('resetting %s', record_name)
        self.set_record(record_name, 'none')
